//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// agent to talk to photo implementation
function PhotoMapPhotoAgent(controller) {
   this.init(controller);
}

PhotoMapPhotoAgent.CACHE_LOCATION = "http://photomap.mozdev.org/notTheCache/";

PhotoMapPhotoAgent.prototype = {
   init : function(controller) {
      this.controller = controller;
      this.preferences = controller.preferences;
      this.createPhotoServices();
   },

   shutdown : function() {
      this.flickrService.saveSearches();
      this.photo23Service.saveSearches();
   },

   getStoredData : function(key) {
      var data = "";
      try {
         var location = PhotoMapPhotoAgent.CACHE_LOCATION + key + ".xml";
         data = PhotoMapUtils.readCacheEntry("HTTP", location);
      } catch(e) {
      }
      return data;
   },

   saveData : function(key, data) {
      var location = PhotoMapPhotoAgent.CACHE_LOCATION + key + ".xml";
      PhotoMapUtils.log("Saving data: [" + data.length + "]");
      PhotoMapUtils.writeCacheEntry("HTTP", location, data);
   },

   createPhotoServices : function() {
      this.flickrService = new PhotoMapFPhoto("flickr", this);
      this.photo23Service = new PhotoMapFPhoto("23", this);
   },


   setupPhotoService : function() {
   },
   
   locationChanged : function(loc, bounds, zoom, force) {
      PhotoMapUtils.callAsync(this.flickrService, this.flickrService.locationChanged, 10, [loc, bounds, zoom, force]);
      PhotoMapUtils.callAsync(this.photo23Service, this.photo23Service.locationChanged, 10, [loc, bounds, zoom, force]);
      // this.flickrService.locationChanged(loc, bounds, zoom, force);
      // this.photo23Service.locationChanged(loc, bounds, zoom, force);
   },

   stop : function() {
      this.flickrService.stop();
      this.photo23Service.stop();
   },

   getTags : function(editInfo) {
      return this.getDefaultPhotoService().getTags(editInfo);
   },

   getDefaultPhotoService : function() {
      var serviceType = this.preferences.getPhotoServiceType();
      if (serviceType == "23") {
         return this.photo23Service;
      } else {
         return this.flickrService;
      }
   },

   needsAuthentication : function() {
      return this.getDefaultPhotoService().needsAuthentication();
   },

   authenticate : function() {
      this.getDefaultPhotoService().authenticate();
   },

   doneAuthenticate : function() {
      this.getDefaultPhotoService().doneAuthenticate();
   },

   getNewAccountUrl : function() {
      return this.getDefaultPhotoService().getNewAccountUrl();
   },

   openWindow : function(url) {
      this.controller.openWindow(url);
   },

   authenticationSuccess : function() {
      this.controller.authenticationSuccess();
   },

   logout : function() {
      this.getDefaultPhotoService().logout();
   },

   getUserName : function() {
      return this.getDefaultPhotoService().getUserName();
   },

   getTotalPictures : function() {
      return this.getDefaultPhotoService().getTotalPictures();
   },

   upload : function(editInfo, callback) {
      this.getDefaultPhotoService().upload(editInfo, callback);
   },

   update : function(editInfo, callback) {
      this.getDefaultPhotoService().update(editInfo, callback);
   },

   photoUpdate : function(key) {
      this.controller.markerUpdate(key);
   },

   // these need to do agregation...
   getByKey : function(key) {
      var photos = {};
      this.addAndMerge(photos, this.flickrService.getByKey(key));
      this.addAndMerge(photos, this.photo23Service.getByKey(key));
      return photos;
   },

   getByLatLon : function(lat, lon) {
      return this.getByKey(Photo.key(lat, lon));
   },

   // this is going to cause problems...
   getAll : function() {
      var photos = {};
      this.addAndMerge(photos, this.flickrService.getAll());
      this.addAndMerge(photos, this.photo23Service.getAll());
      return photos;
   },

   getAllKeys : function() {
      return this.getAll();
   },

   addAndMerge : function(target, source) {
      var mergedItem;
      for (var item in source) {
         if (target[item]) {
            mergedItem = {};
            this.addAndMerge(mergedItem, target[item]);
            this.addAndMerge(mergedItem, source[item]);
            target[item] = mergedItem;
         } else {
            target[item] = source[item];
         }
      }
   },

   getPhotoInfo : function(data) {
      // doesn't really matter which one does it
      var photoInfo = this.getDefaultPhotoService().parsePhotoLocation(data);
      if (photoInfo.provider == "flickr") {
         this.flickrService.getPhotoData(photoInfo);
      } else {
         this.photo23Service.getPhotoData(photoInfo);
      }
   },

   donePhotoData : function(wasError, photo) {
      this.controller.photoLocationInfo(wasError, photo);
   }
};
